@@ -84,7 +84,8 @@ impl Default for Config {
     }
 }
 
-pub async fn run(config: Config, mut api: CommonApi) -> Result<()> {
+pub async fn run(config: &Config, api: &CommonApi) -> Result<()> {
+    let mut actions = api.get_actions().await?;
     api.set_default_actions(&[(MouseButton::Right, None, "next_filter")])
         .await?;
 
@@ -99,12 +100,13 @@ pub async fn run(config: Config, mut api: CommonApi) -> Result<()> {
     let mut filters = config.filters.iter().cycle();
     let mut filter = filters.next().error("`filters` is empty")?;
 
-    let mut notify = Inotify::init().error("Failed to start inotify")?;
+    let notify = Inotify::init().error("Failed to start inotify")?;
     notify
-        .add_watch(&*config.data_location.expand()?, WatchMask::MODIFY)
+        .watches()
+        .add(&*config.data_location.expand()?, WatchMask::MODIFY)
         .error("Failed to watch data location")?;
     let mut updates = notify
-        .event_stream([0; 1024])
+        .into_event_stream([0; 1024])
         .error("Failed to create event stream")?;
 
     loop {
@@ -119,7 +121,7 @@ pub async fn run(config: Config, mut api: CommonApi) -> Result<()> {
         });
 
         widget.set_values(map! {
-            "icon" => Value::icon(api.get_icon("tasks")?),
+            "icon" => Value::icon("tasks"),
             "count" => Value::number(number_of_tasks),
             "filter_name" => Value::text(filter.name.clone()),
         });
@@ -135,8 +137,9 @@ pub async fn run(config: Config, mut api: CommonApi) -> Result<()> {
         select! {
             _ = sleep(config.interval.0) =>(),
             _ = updates.next() => (),
-            event = api.event() => match event {
-                Action(a) if a == "next_filter" => {
+            _ = api.wait_for_update_request() => (),
+            Some(action) = actions.recv() => match action.as_ref() {
+                "next_filter" => {
                     filter = filters.next().unwrap();
                 }
                 _ => (),
